@@ -20,9 +20,11 @@
 #include <stdint.h>
 #include <errno.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <string.h>
 #include <vpn.h>
 #include <sys/socket.h>
+#include <sys/uio.h>
 #include "common.h"
 
 const char* cmd_request_to_str(unsigned _cmd)
@@ -31,18 +33,10 @@ cmd_request_t cmd = _cmd;
 static char tmp[32];
 
 	switch(cmd) {
-	case AUTH_INIT:
-		return "auth init";
-	case AUTH_REINIT:
-		return "auth reinit";
-	case AUTH_REP:
-		return "auth reply";
-	case AUTH_REQ:
-		return "auth request";
+	case AUTH_COOKIE_REP:
+		return "auth cookie reply";
 	case AUTH_COOKIE_REQ:
 		return "auth cookie request";
-	case AUTH_MSG:
-		return "auth msg";
 	case RESUME_STORE_REQ:
 		return "resume data store request";
 	case RESUME_DELETE_REQ:
@@ -59,6 +53,23 @@ static char tmp[32];
 		return "terminate";
 	case CMD_SESSION_INFO:
 		return "session info";
+	case CMD_CLI_STATS:
+		return "cli stats";
+
+	case SM_CMD_AUTH_INIT:
+		return "sm: auth init";
+	case SM_CMD_AUTH_CONT:
+		return "sm: auth cont";
+	case SM_CMD_AUTH_REP:
+		return "sm: auth rep";
+	case SM_CMD_DECRYPT:
+		return "sm: decrypt";
+	case SM_CMD_SIGN:
+		return "sm: sign";
+	case SM_CMD_AUTH_SESSION_CLOSE:
+		return "sm: session close";
+	case SM_CMD_AUTH_SESSION_OPEN:
+		return "sm: session open";
 	default:
 		snprintf(tmp, sizeof(tmp), "unknown (%u)", _cmd);
 		return tmp;
@@ -76,14 +87,16 @@ const uint8_t * p = buf;
 		if (ret == -1) {
 			if (errno != EAGAIN && errno != EINTR)
 				return ret;
+			else
+				ms_sleep(50);
 		}
-		
+
 		if (ret > 0) {
 			left -= ret;
 			p += ret;
 		}
 	}
-	
+
 	return len;
 }
 
@@ -99,13 +112,13 @@ uint8_t * p = buf;
 			if (errno != EAGAIN && errno != EINTR)
 				return ret;
 		}
-		
+
 		if (ret > 0) {
 			left -= ret;
 			p += ret;
 		}
 	}
-	
+
 	return len;
 }
 
@@ -119,7 +132,7 @@ fd_set set;
 
 	tv.tv_sec = sec;
 	tv.tv_usec = 0;
-	
+
 	FD_ZERO(&set);
 	FD_SET(sockfd, &set);
 
@@ -136,17 +149,52 @@ fd_set set;
 		if (ret == -1) {
 			if (errno != EAGAIN && errno != EINTR)
 				return ret;
+		} else 	if (ret == 0 && left != 0) {
+			errno = ENOENT;
+			return -1;
 		}
-		
+
 		if (ret > 0) {
 			left -= ret;
 			p += ret;
 		}
 	}
-	
+
 	return len;
 }
 
+void set_non_block(int fd)
+{
+int val;
+
+	val = fcntl(fd, F_GETFL, 0);
+	fcntl(fd, F_SETFL, val | O_NONBLOCK);
+}
+
+ssize_t recv_timeout(int sockfd, void *buf, size_t len, unsigned sec)
+{
+int ret;
+struct timeval tv;
+fd_set set;
+
+	tv.tv_sec = sec;
+	tv.tv_usec = 0;
+
+	FD_ZERO(&set);
+	FD_SET(sockfd, &set);
+
+	do {
+		ret = select(sockfd + 1, &set, NULL, NULL, &tv);
+	} while (ret == -1 && errno == EINTR);
+
+	if (ret == -1 || ret == 0) {
+		errno = ETIMEDOUT;
+		return -1;
+	}
+
+	return recv(sockfd, buf, len, 0);
+}
+
 int ip_cmp(const struct sockaddr_storage *s1, const struct sockaddr_storage *s2, size_t n)
 {
 	if (((struct sockaddr*)s1)->sa_family == AF_INET) {
@@ -158,32 +206,32 @@ int ip_cmp(const struct sockaddr_storage *s1, const struct sockaddr_storage *s2,
 
 /* returns an allocated string with the mask to apply for the prefix
  */
-char* ipv6_prefix_to_mask(unsigned prefix)
+char* ipv6_prefix_to_mask(void *pool, unsigned prefix)
 {
 	switch (prefix) {
 		case 16:
-			return strdup("FFFF::");
+			return talloc_strdup(pool, "ffff::");
 		case 32:
-			return strdup("FFFF:FFFF::");
+			return talloc_strdup(pool, "ffff:ffff::");
 		case 48:
-			return strdup("FFFF:FFFF:FFFF::");
+			return talloc_strdup(pool, "ffff:ffff:ffff::");
 		case 64:
-			return strdup("FFFF:FFFF:FFFF:FFFF::");
+			return talloc_strdup(pool, "ffff:ffff:ffff:ffff::");
 		case 80:
-			return strdup("FFFF:FFFF:FFFF:FFFF:FFFF::");
+			return talloc_strdup(pool, "ffff:ffff:ffff:ffff:ffff::");
 		case 96:
-			return strdup("FFFF:FFFF:FFFF:FFFF:FFFF:FFFF::");
+			return talloc_strdup(pool, "ffff:ffff:ffff:ffff:ffff:ffff::");
 		case 112:
-			return strdup("FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF::");
+			return talloc_strdup(pool, "ffff:ffff:ffff:ffff:ffff:ffff:ffff::");
 		case 128:
-			return strdup("FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF");
+			return talloc_strdup(pool, "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff");
 		default:
 			return NULL;
 	}
 }
 
 /* Sends message + socketfd */
-int send_socket_msg(int fd, uint8_t cmd, 
+int send_socket_msg(void *pool, int fd, uint8_t cmd, 
 		    int socketfd,
 		    const void* msg, pack_size_func get_size, pack_func pack)
 {
@@ -199,7 +247,7 @@ int send_socket_msg(int fd, uint8_t cmd,
 	int ret;
 
 	memset(&hdr, 0, sizeof(hdr));
-	
+
 	iov[0].iov_base = &cmd;
 	iov[0].iov_len = 1;
 
@@ -212,9 +260,9 @@ int send_socket_msg(int fd, uint8_t cmd,
 	hdr.msg_iovlen = 2;
 
 	if (length > 0) {
-		packed = malloc(length);
+		packed = talloc_size(pool, length);
 		if (packed == NULL) {
-			syslog(LOG_ERR, "%s:%u: memory error", __func__, __LINE__);
+			syslog(LOG_ERR, "%s:%u: memory error", __FILE__, __LINE__);
 			return -1;
 		}
 
@@ -223,7 +271,7 @@ int send_socket_msg(int fd, uint8_t cmd,
 
 		ret = pack(msg, packed);
 		if (ret == 0) {
-			syslog(LOG_ERR, "%s:%u: packing error", __func__, __LINE__);
+			syslog(LOG_ERR, "%s:%u: packing error", __FILE__, __LINE__);
 			ret = -1;
 			goto cleanup;
 		}
@@ -234,7 +282,7 @@ int send_socket_msg(int fd, uint8_t cmd,
 	if (socketfd != -1) {
 		hdr.msg_control = control_un.control;
 		hdr.msg_controllen = sizeof(control_un.control);
-	
+
 		cmptr = CMSG_FIRSTHDR(&hdr);
 		cmptr->cmsg_len = CMSG_LEN(sizeof(int));
 		cmptr->cmsg_level = SOL_SOCKET;
@@ -245,22 +293,22 @@ int send_socket_msg(int fd, uint8_t cmd,
 	ret = sendmsg(fd, &hdr, 0);
 	if (ret < 0) {
 		int e = errno;
-		syslog(LOG_ERR, "%s:%u: %s", __func__, __LINE__, strerror(e));
+		syslog(LOG_ERR, "%s:%u: %s", __FILE__, __LINE__, strerror(e));
 	}
 
 cleanup:
-	free(packed);
+	talloc_free(packed);
 	return ret;
 
 }
 
-int send_msg(int fd, uint8_t cmd, 
+int send_msg(void *pool, int fd, uint8_t cmd, 
 	    const void* msg, pack_size_func get_size, pack_func pack)
 {
-	return send_socket_msg(fd, cmd, -1, msg, get_size, pack);
+	return send_socket_msg(pool, fd, cmd, -1, msg, get_size, pack);
 }
 
-int recv_socket_msg(int fd, uint8_t cmd, 
+int recv_socket_msg(void *pool, int fd, uint8_t cmd, 
 		     int* socketfd, void** msg, unpack_func unpack)
 {
 	struct iovec iov[3];
@@ -274,6 +322,7 @@ int recv_socket_msg(int fd, uint8_t cmd,
 	} control_un;
 	struct cmsghdr  *cmptr;
 	int ret;
+	PROTOBUF_ALLOCATOR(pa, pool);
 
 	iov[0].iov_base = &rcmd;
 	iov[0].iov_len = 1;
@@ -288,29 +337,30 @@ int recv_socket_msg(int fd, uint8_t cmd,
 	hdr.msg_control = control_un.control;
 	hdr.msg_controllen = sizeof(control_un.control);
 
+	/* FIXME: Add a timeout here */
 	do {
-		ret = recvmsg( fd, &hdr, 0);
+		ret = recvmsg(fd, &hdr, 0);
 	} while (ret == -1 && errno == EINTR);
 	if (ret == -1) {
 		int e = errno;
-		syslog(LOG_ERR, "%s:%u: recvmsg: %s", __func__, __LINE__, strerror(e));
+		syslog(LOG_ERR, "%s:%u: recvmsg: %s", __FILE__, __LINE__, strerror(e));
 		return ERR_BAD_COMMAND;
 	}
 
 	if (ret == 0) {
-		syslog(LOG_ERR, "%s:%u: recvmsg returned zero", __func__, __LINE__);
-		return ERR_WORKER_TERMINATED;
+		syslog(LOG_ERR, "%s:%u: recvmsg returned zero", __FILE__, __LINE__);
+		return ERR_PEER_TERMINATED;
 	}
-	
+
 	if (rcmd != cmd) {
 		return ERR_BAD_COMMAND;
 	}
-	
+
 	/* try to receive socket (if any) */
 	if (socketfd != NULL) {
 		if ( (cmptr = CMSG_FIRSTHDR(&hdr)) != NULL && cmptr->cmsg_len == CMSG_LEN(sizeof(int))) {
 			if (cmptr->cmsg_level != SOL_SOCKET || cmptr->cmsg_type != SCM_RIGHTS) {
-				syslog(LOG_ERR, "%s:%u: recvmsg returned invalid msg type", __func__, __LINE__);
+				syslog(LOG_ERR, "%s:%u: recvmsg returned invalid msg type", __FILE__, __LINE__);
 				return ERR_BAD_COMMAND;
 			}
 
@@ -321,7 +371,7 @@ int recv_socket_msg(int fd, uint8_t cmd,
 	}
 
 	if (length > 0) {
-		data = malloc(length);
+		data = talloc_size(pool, length);
 		if (data == NULL) {
 			ret = ERR_MEM;
 			goto cleanup;
@@ -330,30 +380,40 @@ int recv_socket_msg(int fd, uint8_t cmd,
 		ret = force_read(fd, data, length);
 		if (ret < length) {
 			int e = errno;
-			syslog(LOG_ERR, "%s:%u: recvmsg: %s", __func__, __LINE__, strerror(e));
+			syslog(LOG_ERR, "%s:%u: recvmsg: %s", __FILE__, __LINE__, strerror(e));
 			ret = ERR_BAD_COMMAND;
 			goto cleanup;
 		}
 
-		*msg = unpack(NULL, length, data);
+		*msg = unpack(&pa, length, data);
 		if (*msg == NULL) {
-			syslog(LOG_ERR, "%s:%u: unpacking error", __func__, __LINE__);
+			syslog(LOG_ERR, "%s:%u: unpacking error", __FILE__, __LINE__);
 			ret = ERR_MEM;
 			goto cleanup;
 		}
 	}
-	
+
 	ret = 0;
 
 cleanup:
-	free(data);
+	talloc_free(data);
 	if (ret < 0 && socketfd != NULL && *socketfd != -1)
 		close(*socketfd);
 	return ret;
 }
 
-int recv_msg(int fd, uint8_t cmd, 
+int recv_msg(void *pool, int fd, uint8_t cmd, 
 		void** msg, unpack_func unpack)
 {
-	return recv_socket_msg(fd, cmd, NULL, msg, unpack);
+	return recv_socket_msg(pool, fd, cmd, NULL, msg, unpack);
+}
+
+void _talloc_free2(void *ctx, void *ptr)
+{
+	talloc_free(ptr);
+}
+
+void *_talloc_size2(void *ctx, size_t size)
+{
+	return talloc_size(ctx, size);
 }
