@@ -47,6 +47,26 @@ static void bignum_add (uint8_t * num, unsigned size, unsigned step)
     }
 }
 
+void ip_from_seed(uint8_t *seed, unsigned seed_size,
+		void *ip, size_t ip_size)
+{
+	uint8_t digest[20];
+	int ret;
+
+	if (ip_size > sizeof(digest)) {
+		syslog(LOG_ERR, "too large IP!");
+		abort();
+	}
+
+	ret = gnutls_hash_fast(GNUTLS_DIG_SHA1, seed, seed_size, digest);
+	if (ret < 0) {
+		syslog(LOG_ERR, "cannot hash: %s", strerror(ret));
+		abort();
+	}
+
+	memcpy(ip, digest, ip_size);
+
+}
 
 void ip_lease_deinit(struct ip_lease_db_st* db)
 {
@@ -55,11 +75,12 @@ struct htable_iter iter;
 
 	cache = htable_first(&db->ht, &iter);
 	while(cache != NULL) {
-		free(cache);
+		/* disable the destructor */
+		cache->db = NULL;
+		talloc_free(cache);
 		
 		cache = htable_next(&db->ht, &iter);
 	}
-	
 	htable_clear(&db->ht);
 	
 	return;
@@ -102,6 +123,11 @@ struct ip_lease_st t;
 	return 0;
 }
 
+void steal_ip_leases(struct proc_st* proc, struct proc_st *thief)
+{
+	thief->ipv4 = talloc_move(thief, &proc->ipv4);
+	thief->ipv6 = talloc_move(thief, &proc->ipv6);
+}
 
 #define MAX_IP_TRIES 16
 
@@ -141,9 +167,10 @@ int get_ipv4_lease(main_server_st* s, struct proc_st* proc)
 			return -1;
 		}
 
-		proc->ipv4 = calloc(1, sizeof(*proc->ipv4));
+		proc->ipv4 = talloc_zero(proc, struct ip_lease_st);
 		if (proc->ipv4 == NULL)
 			return ERR_MEM;
+		proc->ipv4->db = &s->ip_leases;
 
 		/* mask the network (just in case it is wrong) */
 		for (i=0;i<sizeof(struct in_addr);i++)
@@ -166,17 +193,14 @@ int get_ipv4_lease(main_server_st* s, struct proc_st* proc)
 				goto fail;
 			}
 			if (max_loops == MAX_IP_TRIES) {
-				if (proc->seeds_are_set) {
-					memcpy(SA_IN_U8_P(&rnd), proc->ipv4_seed, 4);
-				} else {
-					uint32_t t = hash_any(proc->username, strlen(proc->username), 0);
-					memcpy(SA_IN_U8_P(&rnd), &t, 4);
-				}
-			} else
-				gnutls_rnd(GNUTLS_RND_NONCE, SA_IN_U8_P(&rnd), sizeof(struct in_addr));
+				memcpy(SA_IN_U8_P(&rnd), proc->ipv4_seed, 4);
+			} else {
+				ip_from_seed(SA_IN_U8_P(&rnd), sizeof(struct in_addr),
+						SA_IN_U8_P(&rnd), sizeof(struct in_addr));
+			}
 			max_loops--;
 
-        		if (SA_IN_U8_P(&rnd)[3] == 255) /* broadcast */
+        		if (SA_IN_U8_P(&rnd)[3] == 255 || SA_IN_U8_P(&rnd)[3] == 254) /* avoid broadcast */
 	        		bignum_add(SA_IN_U8_P(&rnd), sizeof(struct in_addr), 1);
 
 			/* Mask the random number with the netmask */
@@ -236,7 +260,7 @@ int get_ipv4_lease(main_server_st* s, struct proc_st* proc)
 	return 0;
 
 fail:
-	free(proc->ipv4);
+	talloc_free(proc->ipv4);
 	proc->ipv4 = NULL;
 
 	return ret;
@@ -277,9 +301,10 @@ int get_ipv6_lease(main_server_st* s, struct proc_st* proc)
 			return -1;
 		}
 		
-		proc->ipv6 = calloc(1, sizeof(*proc->ipv6));
+		proc->ipv6 = talloc_zero(proc, struct ip_lease_st);
 		if (proc->ipv6 == NULL)
 			return ERR_MEM;
+		proc->ipv6->db = &s->ip_leases;
 
 		/* mask the network */
 		for (i=0;i<sizeof(struct in6_addr);i++)
@@ -289,10 +314,10 @@ int get_ipv6_lease(main_server_st* s, struct proc_st* proc)
 
        		memcpy(&tmp, &network, sizeof(tmp));
         	((struct sockaddr_in6*)&tmp)->sin6_family = AF_INET6;
-        	((struct sockaddr_in6*)&tmp)->sin6_port = AF_INET6;
+        	((struct sockaddr_in6*)&tmp)->sin6_port = 0;
 
         	((struct sockaddr_in6*)&rnd)->sin6_family = AF_INET6;
-        	((struct sockaddr_in6*)&rnd)->sin6_port = AF_INET6;
+        	((struct sockaddr_in6*)&rnd)->sin6_port = 0;
 
 		do {
 			if (max_loops == 0) {
@@ -303,14 +328,11 @@ int get_ipv6_lease(main_server_st* s, struct proc_st* proc)
 			
 			if (max_loops == MAX_IP_TRIES) {
 				memset(SA_IN6_U8_P(&rnd), 0, sizeof(struct in6_addr));
-				if (proc->seeds_are_set) {
-					memcpy(SA_IN6_U8_P(&rnd)+sizeof(struct in6_addr)-5, proc->ipv4_seed, 4);
-				} else {
-					uint32_t t = hash_any(proc->username, strlen(proc->username), 0);
-					memcpy(SA_IN6_U8_P(&rnd)+sizeof(struct in6_addr)-5, &t, 4);
-				}
-			} else
-				gnutls_rnd(GNUTLS_RND_NONCE, SA_IN6_U8_P(&rnd), sizeof(struct in6_addr));
+				memcpy(SA_IN6_U8_P(&rnd)+sizeof(struct in6_addr)-5, proc->ipv4_seed, 4);
+			} else {
+				ip_from_seed(SA_IN6_U8_P(&rnd), sizeof(struct in6_addr),
+						SA_IN6_U8_P(&rnd), sizeof(struct in6_addr));
+			}
 			max_loops--;
 			
 			/* Mask the random number with the netmask */
@@ -335,7 +357,8 @@ int get_ipv6_lease(main_server_st* s, struct proc_st* proc)
         		memcpy(&proc->ipv6->lip, &rnd, proc->ipv6->lip_len);
 
         		/* RIP = LIP + 1 */
-        		memcpy(&tmp, &proc->ipv6->lip, proc->ipv6->rip_len);
+        		memcpy(&tmp, &proc->ipv6->lip, sizeof(tmp));
+        		proc->ipv6->rip_len = proc->ipv6->lip_len;
 
 	        	bignum_add(SA_IN6_U8_P(&tmp), sizeof(struct in6_addr), 1);
 
@@ -347,7 +370,6 @@ int get_ipv6_lease(main_server_st* s, struct proc_st* proc)
 				continue;
 			}
 
-        		proc->ipv6->rip_len = sizeof(struct sockaddr_in6);
         		memcpy(&proc->ipv6->rip, &tmp, proc->ipv6->rip_len);
 
         		/* mask the last IP with the netmask */
@@ -369,13 +391,22 @@ int get_ipv6_lease(main_server_st* s, struct proc_st* proc)
 
 	return 0;
 fail:
-	free(proc->ipv6);
+	talloc_free(proc->ipv6);
 	proc->ipv6 = NULL;
 
 	return ret;
 
 }
 
+static
+int unref_ip_lease(struct ip_lease_st * lease)
+{
+	if (lease->db) {
+		htable_del(&lease->db->ht, rehash(lease, NULL), lease);
+	}
+	return 0;
+}
+
 int get_ip_leases(main_server_st* s, struct proc_st* proc)
 {
 int ret;
@@ -391,6 +422,7 @@ char buf[128];
 				mslog(s, proc, LOG_ERR, "could not add IPv4 lease to hash table.");
 				return -1;
 			}
+			talloc_set_destructor(proc->ipv4, unref_ip_lease);
 		}
 	}
 
@@ -404,6 +436,7 @@ char buf[128];
 				mslog(s, proc, LOG_ERR, "could not add IPv6 lease to hash table.");
 				return -1;
 			}
+			talloc_set_destructor(proc->ipv6, unref_ip_lease);
 		}
 	}
 
@@ -419,20 +452,23 @@ char buf[128];
 	if (proc->ipv6)
 		mslog(s, proc, LOG_INFO, "assigned IPv6 to '%s': %s", proc->username,
 			human_addr((void*)&proc->ipv6->rip, proc->ipv6->rip_len, buf, sizeof(buf)));
-		
+
 	return 0;
 }
 
 void remove_ip_leases(main_server_st* s, struct proc_st* proc)
 {
 	if (proc->ipv4) {
-		htable_del(&s->ip_leases.ht, rehash(proc->ipv4, NULL), proc->ipv4);
-		free(proc->ipv4);
+		talloc_free(proc->ipv4);
 		proc->ipv4 = NULL;
 	}
 	if (proc->ipv6) {
-		htable_del(&s->ip_leases.ht, rehash(proc->ipv6, NULL), proc->ipv6);
-		free(proc->ipv6);
+		talloc_free(proc->ipv6);
 		proc->ipv6 = NULL;
 	}
 }
+
+void remove_ip_lease(main_server_st* s, struct ip_lease_st * lease)
+{
+	talloc_free(lease);
+}
