@@ -43,6 +43,7 @@
 #include <sec-mod.h>
 #include <route-add.h>
 #include <ip-lease.h>
+#include <proc-search.h>
 #include <ipc.pb-c.h>
 #include <script-list.h>
 #include <main-sup-config.h>
@@ -256,8 +257,9 @@ void remove_proc(main_server_st * s, struct proc_st *proc, unsigned k)
 		kill(proc->pid, SIGTERM);
 
 	remove_from_script_list(s, proc);
-	if (proc->username[0] != 0)
+	if (proc->status == PS_AUTH_COMPLETED) {
 		user_disconnected(s, proc);
+	}
 
 	/* close any pending sessions */
 	if (s->config->session_control != 0 && proc->active_sid) {
@@ -294,6 +296,7 @@ void remove_proc(main_server_st * s, struct proc_st *proc, unsigned k)
 	}
 
 	close_tun(s, proc);
+	proc_table_del(s, proc);
 
 	talloc_free(proc);
 }
