@@ -56,6 +56,7 @@
 #include <route-add.h>
 #include <worker.h>
 #include <cookies.h>
+#include <proc-search.h>
 #include <tun.h>
 #include <grp.h>
 #include <ip-lease.h>
@@ -625,6 +626,7 @@ void clear_lists(main_server_st *s)
 
 	tls_cache_deinit(&s->tls_db);
 	ip_lease_deinit(&s->ip_leases);
+	proc_table_deinit(s);
 	ctl_handler_deinit(s);
 	cookie_db_deinit(&s->cookies);
 }
@@ -664,7 +666,7 @@ static int forward_udp_to_owner(main_server_st* s, struct listener_st *listener)
 {
 int ret, e;
 struct sockaddr_storage cli_addr;
-struct proc_st *ctmp = NULL, *proc_to_send = NULL;
+struct proc_st *proc_to_send = NULL;
 socklen_t cli_addr_size;
 uint8_t buffer[1024];
 char tbuf[64];
@@ -672,7 +674,7 @@ uint8_t  *session_id = NULL;
 int session_id_size = 0;
 ssize_t buffer_size;
 int connected = 0;
-int match_ip_only = 0, matching_ips;
+int match_ip_only = 0;
 time_t now;
 
 	/* first receive from the correct client and connect socket */
@@ -687,26 +689,29 @@ time_t now;
 	
 	/* obtain the session id */
 	if (buffer_size < RECORD_PAYLOAD_POS+HANDSHAKE_SESSION_ID_POS+GNUTLS_MAX_SESSION_ID+2) {
-		mslog(s, NULL, LOG_INFO, "%s: too short packet",
+		mslog(s, NULL, LOG_INFO, "%s: too short UDP packet",
 		      human_addr((struct sockaddr*)&cli_addr, cli_addr_size, tbuf, sizeof(tbuf)));
 		goto fail;
 	}
 
 	/* check version */
-	mslog(s, NULL, LOG_DEBUG, "new DTLS session from %s (record v%u.%u, hello v%u.%u)", 
-		human_addr((struct sockaddr*)&cli_addr, cli_addr_size, tbuf, sizeof(tbuf)),
-		(unsigned int)buffer[1], (unsigned int)buffer[2],
-		(unsigned int)buffer[RECORD_PAYLOAD_POS], (unsigned int)buffer[RECORD_PAYLOAD_POS+1]);
+	if (buffer[0] == 22) {
+		mslog(s, NULL, LOG_DEBUG, "new DTLS session from %s (record v%u.%u, hello v%u.%u)", 
+			human_addr((struct sockaddr*)&cli_addr, cli_addr_size, tbuf, sizeof(tbuf)),
+			(unsigned int)buffer[1], (unsigned int)buffer[2],
+			(unsigned int)buffer[RECORD_PAYLOAD_POS], (unsigned int)buffer[RECORD_PAYLOAD_POS+1]);
+	}
 
 	if (buffer[1] != 254 && (buffer[1] != 1 && buffer[2] != 0) &&
 		buffer[RECORD_PAYLOAD_POS] != 254 && (buffer[RECORD_PAYLOAD_POS] != 0 && buffer[RECORD_PAYLOAD_POS+1] != 0)) {
-		mslog(s, NULL, LOG_INFO, "%s: unknown DTLS version: %u.%u", 
+		mslog(s, NULL, LOG_INFO, "%s: unknown DTLS record version: %u.%u", 
 		      human_addr((struct sockaddr*)&cli_addr, cli_addr_size, tbuf, sizeof(tbuf)),
 		      (unsigned)buffer[1], (unsigned)buffer[2]);
 		goto fail;
 	}
+
 	if (buffer[0] != 22) {
-		mslog(s, NULL, LOG_INFO, "%s: unexpected DTLS content type: %u",
+		mslog(s, NULL, LOG_DEBUG, "%s: unexpected DTLS content type: %u; possibly a firewall disassociated a UDP session",
 		      human_addr((struct sockaddr*)&cli_addr, cli_addr_size, tbuf, sizeof(tbuf)),
 		      (unsigned int)buffer[0]);
 		/* Here we received a non-client hello packet. It may be that
@@ -715,6 +720,10 @@ time_t now;
 		 * the IP address and forward the socket.
 		 */
 		match_ip_only = 1;
+
+		/* don't bother IP matching when the listen-clear-file is in use */
+		if (s->config->unix_conn_file)
+			goto fail;
 	} else {
 		/* read session_id */
 		session_id_size = buffer[RECORD_PAYLOAD_POS+HANDSHAKE_SESSION_ID_POS];
@@ -723,34 +732,18 @@ time_t now;
 
 	/* search for the IP and the session ID in all procs */
 	now = time(0);
-	matching_ips = 0;
 
-	list_for_each(&s->proc_list.head, ctmp, list) {
-		if (match_ip_only == 0 && session_id_size == ctmp->dtls_session_id_size &&
-			memcmp(session_id, ctmp->dtls_session_id, session_id_size) == 0) {
-
-			proc_to_send = ctmp;
-			break;
-		} else if (match_ip_only != 0 && cli_addr_size == ctmp->remote_addr_len &&
-			memcmp(SA_IN_P_GENERIC(&cli_addr, cli_addr_size), 
-				SA_IN_P_GENERIC(&ctmp->remote_addr, ctmp->remote_addr_len),
-				SA_IN_SIZE(ctmp->remote_addr_len)) == 0) {
-			matching_ips++;
-			proc_to_send = ctmp;
-		}
+	if (match_ip_only == 0) {
+		proc_to_send = proc_search_sid(s, session_id, session_id_size);
+	} else {
+		proc_to_send = proc_search_ip(s, &cli_addr, cli_addr_size);
 	}
 
 	if (proc_to_send != 0) {
 		UdpFdMsg msg = UDP_FD_MSG__INIT;
 
-		if (matching_ips > 1) {
-			mslog(s, proc_to_send, LOG_INFO, "cannot associate with a client; more than a single clients from %s",
-			      human_addr((struct sockaddr*)&cli_addr, cli_addr_size, tbuf, sizeof(tbuf)));
-			goto fail;
-		}
-
 		if (now - proc_to_send->udp_fd_receive_time <= UDP_FD_RESEND_TIME) {
-			mslog(s, proc_to_send, LOG_INFO, "received UDP connection too soon from %s",
+			mslog(s, proc_to_send, LOG_DEBUG, "received UDP connection too soon from %s",
 			      human_addr((struct sockaddr*)&cli_addr, cli_addr_size, tbuf, sizeof(tbuf)));
 			goto fail;
 		}
@@ -921,6 +914,7 @@ int main(int argc, char** argv)
 	tls_cache_init(s, &s->tls_db);
 	cookie_db_init(s, &s->cookies);
 	ip_lease_init(&s->ip_leases);
+	proc_table_init(s);
 
 	sigemptyset(&blockset);
 	sigemptyset(&emptyset);
@@ -1178,10 +1172,7 @@ fork_failed:
 					ms_sleep(s->config->rate_limit_ms);
 			} else if (set && ltmp->sock_type == SOCK_TYPE_UDP) {
 				/* connection on UDP port */
-				ret = forward_udp_to_owner(s, ltmp);
-				if (ret < 0) {
-					mslog(s, NULL, LOG_INFO, "could not determine the owner of received UDP packet");
-				}
+				forward_udp_to_owner(s, ltmp);
 
 				if (s->config->rate_limit_ms > 0)
 					ms_sleep(s->config->rate_limit_ms);
