@@ -22,6 +22,7 @@
 
 #include <cookies.h>
 #include <gnutls/abstract.h>
+#include <ccan/htable/htable.h>
 
 typedef struct sec_mod_st {
 	gnutls_datum_t dcookie_key; /* the key to generate cookies */
@@ -30,8 +31,8 @@ typedef struct sec_mod_st {
 	struct cfg_st *config;
 	gnutls_privkey_t *key;
 	unsigned key_size;
-	void *client_db;
-	void *ban_db;
+	struct htable *client_db;
+	struct htable *ban_db;
 
 	int fd;
 } sec_mod_st;
@@ -62,19 +63,24 @@ typedef struct client_entry_st {
 	time_t time;
 } client_entry_st;
 
-void *sec_mod_client_db_init(void *pool);
-void sec_mod_client_db_deinit(void *db);
-unsigned sec_mod_client_db_elems(void *_db);
-client_entry_st * new_client_entry(void *_db, const char *ip);
-client_entry_st * find_client_entry(void *_db, uint8_t sid[SID_SIZE]);
-void del_client_entry(void *_db, client_entry_st * e);
-void cleanup_client_entries(void *_db);
+void *sec_mod_client_db_init(sec_mod_st *sec);
+void sec_mod_client_db_deinit(sec_mod_st *sec);
+unsigned sec_mod_client_db_elems(sec_mod_st *sec);
+client_entry_st * new_client_entry(sec_mod_st *sec, const char *ip);
+client_entry_st * find_client_entry(sec_mod_st *sec, uint8_t sid[SID_SIZE]);
+void del_client_entry(sec_mod_st *sec, client_entry_st * e);
+void cleanup_client_entries(sec_mod_st *sec);
 
 #ifdef __GNUC__
-# define seclog(prio, fmt, ...) \
-	syslog(prio, "sec-mod: "fmt, ##__VA_ARGS__)
+# define seclog(sec, prio, fmt, ...) \
+	if (prio != LOG_DEBUG || sec->config->debug != 0) { \
+		syslog(prio, "sec-mod: "fmt, ##__VA_ARGS__); \
+	}
 #else
-# define seclog syslog
+# define seclog(sec,prio,...) \
+	if (prio != LOG_DEBUG || sec->config->debug != 0) { \
+		 syslog(prio, __VA_ARGS__); \
+	}
 #endif
 
 void sec_auth_init(struct cfg_st *config);
@@ -82,16 +88,16 @@ void sec_auth_init(struct cfg_st *config);
 int handle_sec_auth_init(sec_mod_st *sec, const SecAuthInitMsg * req);
 int handle_sec_auth_cont(sec_mod_st *sec, const SecAuthContMsg * req);
 int handle_sec_auth_session_cmd(sec_mod_st * sec, const SecAuthSessionMsg * req, unsigned cmd);
-void sec_auth_user_deinit(client_entry_st * e);
+void sec_auth_user_deinit(sec_mod_st * sec, client_entry_st * e);
 
 void sec_mod_server(void *main_pool, struct cfg_st *config, const char *socket_file,
 		    uint8_t cookie_key[COOKIE_KEY_SIZE]);
 
-void cleanup_banned_entries(void *_db);
-unsigned check_if_banned(void *_db, const char *ip);
-void add_ip_to_ban_list(void *_db, const char *ip, time_t reenable_time);
-void *sec_mod_ban_db_init(void *pool);
-void sec_mod_ban_db_deinit(void *_db);
-unsigned sec_mod_ban_db_elems(void *_db);
+void cleanup_banned_entries(sec_mod_st *sec);
+unsigned check_if_banned(sec_mod_st *sec, const char *ip);
+void add_ip_to_ban_list(sec_mod_st *sec, const char *ip, time_t reenable_time);
+void *sec_mod_ban_db_init(sec_mod_st *sec);
+void sec_mod_ban_db_deinit(sec_mod_st *sec);
+unsigned sec_mod_ban_db_elems(sec_mod_st *sec);
 
 #endif
