@@ -98,9 +98,8 @@ extern int syslog_open;
 
 #define MAX_AUTH_SECS 40
 #define MAX_CIPHERSUITE_NAME 64
-#define MAX_DTLS_CIPHERSUITE_NAME 24
 #define MAX_MSG_SIZE 256
-#define SID_SIZE 12
+#define SID_SIZE 16
 
 typedef enum {
 	AUTH_COOKIE_REP = 2,
@@ -189,6 +188,7 @@ struct cfg_st {
 	char *name; /* server name */
 	unsigned int port;
 	unsigned int udp_port;
+	unsigned int is_dyndns;
 	char* unix_conn_file;
 
 	char *pin_file;
