@@ -57,10 +57,6 @@
 
 #define MIN_MTU(ws) (((ws)->vinfo.ipv6!=NULL)?1281:257)
 
-/* after that time (secs) of inactivity in the UDP part, connection switches to 
- * TCP (if activity occurs there).
- */
-#define UDP_SWITCH_TIME 15
 #define PERIODIC_CHECK_TIME 30
 
 /* The number of DPD packets a client skips before he's kicked */
@@ -191,7 +187,7 @@ static const dtls_ciphersuite_st ciphersuites[] = {
 	{
 	 .oc_name = CS_AES128_GCM,
 	 .gnutls_name =
-	 "NONE:+VERS-DTLS1.2:+COMP-NULL:+AES-128-GCM:+AEAD:+RSA:%COMPAT:%DISABLE_SAFE_RENEGOTIATION:+SIGN-ALL",
+	 "NONE:+VERS-DTLS1.2:+COMP-NULL:+AES-128-GCM:+AEAD:+RSA:%COMPAT:+SIGN-ALL",
 	 .gnutls_version = GNUTLS_DTLS1_2,
 	 .gnutls_mac = GNUTLS_MAC_AEAD,
 	 .gnutls_cipher = GNUTLS_CIPHER_AES_128_GCM,
@@ -199,7 +195,7 @@ static const dtls_ciphersuite_st ciphersuites[] = {
 	{
 	 .oc_name = CS_AES256_GCM,
 	 .gnutls_name =
-	 "NONE:+VERS-DTLS1.2:+COMP-NULL:+AES-256-GCM:+AEAD:+RSA:%COMPAT:%DISABLE_SAFE_RENEGOTIATION:+SIGN-ALL",
+	 "NONE:+VERS-DTLS1.2:+COMP-NULL:+AES-256-GCM:+AEAD:+RSA:%COMPAT:+SIGN-ALL",
 	 .gnutls_version = GNUTLS_DTLS1_2,
 	 .gnutls_mac = GNUTLS_MAC_AEAD,
 	 .gnutls_cipher = GNUTLS_CIPHER_AES_256_GCM,
@@ -209,7 +205,7 @@ static const dtls_ciphersuite_st ciphersuites[] = {
 	{
 	 .oc_name = "AES128-SHA",
 	 .gnutls_name =
-	 "NONE:+VERS-DTLS0.9:+COMP-NULL:+AES-128-CBC:+SHA1:+RSA:%COMPAT:%DISABLE_SAFE_RENEGOTIATION",
+	 "NONE:+VERS-DTLS0.9:+COMP-NULL:+AES-128-CBC:+SHA1:+RSA:%COMPAT",
 	 .gnutls_version = GNUTLS_DTLS0_9,
 	 .gnutls_mac = GNUTLS_MAC_SHA1,
 	 .gnutls_cipher = GNUTLS_CIPHER_AES_128_CBC,
@@ -218,7 +214,7 @@ static const dtls_ciphersuite_st ciphersuites[] = {
 	{
 	 .oc_name = "DES-CBC3-SHA",
 	 .gnutls_name =
-	 "NONE:+VERS-DTLS0.9:+COMP-NULL:+3DES-CBC:+SHA1:+RSA:%COMPAT:%DISABLE_SAFE_RENEGOTIATION",
+	 "NONE:+VERS-DTLS0.9:+COMP-NULL:+3DES-CBC:+SHA1:+RSA:%COMPAT",
 	 .gnutls_version = GNUTLS_DTLS0_9,
 	 .gnutls_mac = GNUTLS_MAC_SHA1,
 	 .gnutls_cipher = GNUTLS_CIPHER_3DES_CBC,
@@ -233,6 +229,9 @@ static void value_check(struct worker_st *ws, struct http_req_st *req)
 	size_t nlen, value_length;
 	char *token, *value;
 	char *str, *p;
+	const dtls_ciphersuite_st *cand = NULL;
+	gnutls_cipher_algorithm_t want_cipher;
+	gnutls_mac_algorithm_t want_mac;
 
 	if (req->value.length <= 0)
 		return;
@@ -296,6 +295,14 @@ static void value_check(struct worker_st *ws, struct http_req_st *req)
 		break;
 
 	case HEADER_DTLS_CIPHERSUITE:
+		if (ws->session != NULL) {
+			want_mac = gnutls_mac_get(ws->session);
+			want_cipher = gnutls_cipher_get(ws->session);
+		} else {
+			want_mac = -1;
+			want_cipher = -1;
+		}
+
 		req->selected_ciphersuite = NULL;
 
 		str = (char *)value;
@@ -304,17 +311,25 @@ static void value_check(struct worker_st *ws, struct http_req_st *req)
 			     i < sizeof(ciphersuites) / sizeof(ciphersuites[0]);
 			     i++) {
 				if (strcmp(token, ciphersuites[i].oc_name) == 0) {
-					if (req->selected_ciphersuite == NULL ||
-					    req->
-					    selected_ciphersuite->server_prio <
+					if (cand == NULL ||
+					    cand->server_prio <
 					    ciphersuites[i].server_prio) {
-						req->selected_ciphersuite =
+						cand =
 						    &ciphersuites[i];
+
+						/* if our candidate matches the TLS session
+						 * ciphersuite, we are finished */
+						if (want_cipher != -1) {
+							if (want_cipher == cand->gnutls_cipher &&
+							    want_mac == cand->gnutls_mac)
+							    break;
+						}
 					}
 				}
 			}
 			str = NULL;
 		}
+	        req->selected_ciphersuite = cand;
 
 		break;
 
@@ -864,9 +879,9 @@ void session_info_send(worker_st * ws)
 		msg.tls_ciphersuite = gnutls_session_get_desc(ws->session);
 	}
 
-	if (ws->udp_state != UP_DISABLED) {
+	if (ws->udp_state != UP_DISABLED && ws->dtls_session) {
 		msg.dtls_ciphersuite =
-		    (char *)ws->req.selected_ciphersuite->oc_name;
+		    gnutls_session_get_desc(ws->dtls_session);
 	}
 
 	if (ws->req.user_agent[0] != 0) {
@@ -878,6 +893,7 @@ void session_info_send(worker_st * ws)
 			 (pack_func) session_info_msg__pack);
 
 	gnutls_free(msg.tls_ciphersuite);
+	gnutls_free(msg.dtls_ciphersuite);
 }
 
 /* mtu_set: Sets the MTU for the session
@@ -1193,6 +1209,7 @@ static int dtls_mainloop(worker_st * ws, struct timespec *tnow)
 			oclog(ws, LOG_DEBUG,
 			      "DTLS handshake completed (plaintext MTU: %u)\n",
 			      ws->conn_mtu);
+			session_info_send(ws);
 		}
 
 		break;
@@ -1633,6 +1650,13 @@ static int connect_handler(worker_st * ws)
 		     "X-CSTP-Smartcard-Removal-Disconnect: true\r\n");
 	SEND_ERR(ret);
 
+	if (ws->config->is_dyndns != 0) {
+		ret =
+		    cstp_puts(ws,
+			     "X-CSTP-DynDNS: true\r\n");
+		SEND_ERR(ret);
+	}
+
 	if (ws->config->rekey_time > 0) {
 		unsigned method;
 
