@@ -262,6 +262,9 @@ void __attribute__ ((format(printf, 3, 4)))
 # define oclog _oclog
 #endif
 
+void  oclog_hex(const worker_st* ws, int priority,
+		const char *prefix, uint8_t* bin, unsigned bin_size, unsigned b64);
+
 int complete_vpn_info(worker_st * ws,
                     struct vpn_st* vinfo);
 unsigned check_if_default_route(char **routes, unsigned routes_size);
@@ -279,4 +282,10 @@ int send_msg_to_main(worker_st *ws, uint8_t cmd,
 	return send_msg(ws, ws->cmd_fd, cmd, msg, get_size, pack);
 }
 
+/* after that time (secs) of inactivity in the UDP part, connection switches to 
+ * TCP (if activity occurs there).
+ */
+#define UDP_SWITCH_TIME 15
+#define ACTIVE_SESSION_TIMEOUT 30
+
 #endif
